$ErrorActionPreference = 'Stop'
$toolsDir = "$(Split-Path -parent $MyInvocation.MyCommand.Definition)"
$version = '3.49.0-02'
$url64 = 'https://sonatype-download.global.ssl.fastly.net/repository/downloads-prod-group/3/nexus-3.49.0-02-win64.zip'
$checksum64 = 'AAD244BC6F145BDE76EDC35D8EEA44069E093387AE97A38E15D1D1AE24E2B240'
$checksumType64 = 'SHA256'
$validExitCodes = @(0)
Import-Module (Join-Path $toolsDir 'supportingfunctions.psm1')

$name = $env:Computername
$domainName = $env:USERDOMAIN

$hostName = if (-not $name.endswith($domainName)) {
  "$name.$domainName"
}
else {
  $name
}

$packageName = $env:ChocolateyPackagename

$osBits = Get-ProcessorBits

If (!($osBits -eq 64)) {
  Throw "Sonatype Nexus Repository 3.0 and greater only supports 64-bit Windows."
}

$nexusVersionedFolder = "nexus-$version"
$targetFolder = "$env:programdata\nexus"
$extractFolder = "$env:temp\NexusExtract"
$targetDataFolder = "$env:programdata\sonatype-work"
$nexusConfigFile = "$TargetDataFolder\nexus3\etc\nexus.properties"
$serviceName = 'nexus'
$nexusPort = '8081'

$pp = Get-PackageParameters

if ($pp['Fqdn']) {
  $hostname = $pp['Fqdn']
}
  
if ($pp.Port) {
  $nexusPort = $pp.Port
  Write-Host "/Port was used, Nexus will listen on port $NexusPort."
}

If (Test-Path "$env:ProgramFiles\nexus\bin") {
  Throw "Previous version of Nexus 3 installed by setup.exe is present, please uninstall before running this package."
}

If ((Get-Service $serviceName -ErrorAction SilentlyContinue)) {
  Write-Warning "Nexus web app is already present, shutting it down so that we can upgrade it."
  Get-Service $serviceName | Stop-Service -Force
}

if ($pp['BackupSslConfig']) {
  if ($pp['BackupLocation']) {
    Backup-NexusSSL -BackupLocation $pp['BackupLocation']
  }

  else {
    Backup-NexusSSL
  }
}

If (Test-Path "$ExtractFolder") { 
  Remove-Item "$ExtractFolder" -Recurse -Force 
}

$packageArgs = @{
  PackageName    = $packageName
  unzipLocation  = $ExtractFolder
  url64          = $url64
  checksum64     = $checksum64
  checksumType64 = $checksumType64
}

Install-ChocolateyZipPackage @packageArgs

#$VerbosePreference = Continue
Write-Host "Copying files to '$TargetFolder' with overwrite"
If (Test-Path "$TargetFolder") { 
  Copy-Item "$ExtractFolder\$nexusversionedfolder\*" "$TargetFolder" -Force -Recurse
}
else {
  Copy-Item "$ExtractFolder\$nexusversionedfolder" "$TargetFolder" -Force -Recurse
}

If (!(Test-Path "$TargetDataFolder")) {
  Move-Item "$extractfolder\sonatype-work" "$TargetDataFolder"
}
else {
  Write-Warning "`"$TargetDataFolder`" already exists, not overwriting, residual data from previous installs will not be reset."
}

Remove-Item "$ExtractFolder" -Force -Recurse

$processArgs = @{
  ExeToRun       = "$TargetFolder\bin\nexus.exe"
  Statements     = "/install $servicename"
  validExitCodes = $validExitCodes
}

Start-ChocolateyProcessAsAdmin @processArgs

if ($pp['BackupSslConfig']) {
  if ($pp['BackupLocation']) {
    Restore-NexusSSL -BackupLocation $pp['BackupLocation']
  }

  else {
    Restore-NexusSSL
  }
}

#Update Port
If ($nexusPort -ne '8081') {
  If (Test-Path "$NexusConfigFile") {
    Write-Host "Configuring Nexus to listen on port $nexusPort."
    (Get-Content "$NexusConfigFile") -replace "^#\s*application-port=.*$", "application-port=$nexusPort" |
    Set-Content "$NexusConfigFile"
  }
  else {
    Write-Warning "Cannot find `"$NexusConfigFile`", skipping configuring Nexus to listen on port $nexusPort."
  }
}

$service = Start-Service $serviceName -PassThru

If ($Service.Status -ne 'Running') {
  Write-Warning "The Nexus Repository service ($serviceName) did not start."
}
else {
  #Even though windows reports service is ready - web url will not respond until Nexus is actually ready to serve content
  #We need to use this method to collect the port number so we can properly test the website has returned OK.
  $nexusScheme, $portConfigLine = if ($pp['BackupSSLConfig']) {
    # This is to combat Package Internalizer's over-enthusiastic URL matching
    ('http' + 's'), 'application-port-ssl'
  } else {
    'http', 'application-port'
  }

  # As the service is started, this should be present momentarily
  $Timer = [System.Diagnostics.Stopwatch]::StartNew()
  while (-not ($ConfigPresent = Test-Path $NexusConfigFile) -and $Timer.Elapsed.TotalSeconds -le 60) {
    Write-Verbose "Waiting for '$($NexusConfigFile)' to become available ($($Timer.Elapsed.TotalSeconds) seconds waited)..."
    Start-Sleep -Seconds 5
  }

  if (-not $ConfigPresent) {
    Write-Warning "Expected Nexus Config file '$($NexusConfigFile)' is not present."
    $nexusPath = '/'
  } else {
    $nexusPort = (Get-Content $NexusConfigFile | Where-Object {
      $_ -match $portConfigLine
    }).Split('=')[-1]

    $nexusPath = (Get-Content $NexusConfigFile | Where-Object {
      $_ -match "nexus-context-path"
    }).Split("=")[-1]
  }

  $nexusUri = "$($nexusScheme)://$($hostname):$($nexusPort)$($nexusPath)"

  Write-Host "Waiting on Nexus Web UI to be available at '$($nexusUri)'"
  while ($response.StatusCode -ne '200' -and $Timer.Elapsed.TotalMinutes -lt 3) {
    try {
      $response = Invoke-WebRequest -Uri $nexusUri -UseBasicParsing
    } catch {
      Write-Verbose "Waiting on Nexus Web UI to be available at '$($nexusUri)'"
      Start-Sleep -Seconds 1
    }
  }

  $Timer = $null

  if ($Response.StatusCode -eq '200') {
    Write-Host "Nexus is ready!"
  } else {
    Write-Error "Nexus did not respond to requests at '$($nexusUri)' within 3 minutes of the service being started."
  }
}
$generatedAdminPasswordFile = "$targetDataFolder\nexus3\admin.password"
$outputString = @"

*******************************************************************************************
*
*  You MAY receive the error 'localhost refused to connect.' until Nexus is fully started.
*
*  For new installs, you must login as admin to complete some setup steps
*  You can manage the repository by typing 'start http://$($hostname):$NexusPort'
*
*  The default user is 'admin'
*  ADMIN PASSWORD:
*    NEW INSTALLS: The password generated for your instance is recorded
*       in '$generatedAdminPasswordFile'
*    UPGRADES/REINSTALLS: If you upgraded (or uninstalled and reinstalled) without cleaning
*      up $targetDataFolder - the password will be the same as it was before and the password file
*      will not exist.
*    RESET PASSWORD WITH INSTALL: Uninstall Nexus and remove the directory '$targetDataFolder'
*      and then reinstall.  This time a password file will be generated.
*
*  Nexus availability is controlled via the service `"$servicename`"
*  Use the following command to open port $NexusPort for access from off this machine (one line):
*   netsh advfirewall firewall add rule name=`"Nexus Repository`" dir=in action=allow
*   protocol=TCP localport=$NexusPort
*******************************************************************************************
"@

Write-Warning $outputString
